#!/usr/bin/env python

#
#
#    Copyright (C) 2007 by Michal Vyskocil
#    (michal dot vyskocil at gmail dot com)
#
#    This program is free software; you can redistribute it and#or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the
#    Free Software Foundation, Inc.,
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#
#   Simple XML Viewer for KDE
#   depends on
#       - KDE 3.x
#       - Python
#       - Qt
#       - PyQt
#       - PyKDE
#

import sys, os

from qt import QPopupMenu, SIGNAL, SLOT, QLabel, QIconSet, QListViewItem, QListViewItemIterator, QScrollView, QRect, QCString, Qt

from kdecore import KApplication, KCmdLineArgs, KAboutData, i18n, KIcon, KIconLoader, KShortcut
from kdeui import KMainWindow, KMessageBox, KStdAction, KAction, KToggleAction, KFontSizeAction, KFontAction, KRadioAction,\
                  KActionSeparator, KListView

from kfile import KFileDialog

from xml.sax import saxutils
from xml.sax import make_parser
from xml.sax.handler import feature_namespaces

import string

class Stack:
#{{{
    def __init__(self, stack=[]):
        self.__st = []
        if stack != []: self.__st = stack

    def push(self, item):
        self.__st.append(item)

    def pop(self):
        return self.__st.pop()

    def peek(self):
        return self.__st[-1:][0]

    def empty(self):
        return len(self.__st) == 0

    def stack(self):
        return self.__st

#}}}


class ProcessXML(saxutils.DefaultHandler):
#{{{
    @staticmethod
    def normalize(text):
        "Remove redundant whitespace from a string"
        return ' '.join(text.split())

    def __init__(self, tree, xml):
        self.tree = tree

        self.xml = xml.__str__()
        self.stack = Stack()

        self.stack.push(self.tree)

        self.parser = make_parser()
        self.parser.setFeature(feature_namespaces, True)
        self.parser.setContentHandler(self)

    def parse(self):
        self.parser.parse(self.xml)

    def startElementNS(self, (uri, localname), name, attrs):
        ret = "%s " % (name)
        for i in attrs.keys():
            ret += "%s='%s ' " % (i[1], ProcessXML.normalize(attrs[i]))
        
        parent = QListViewItem(self.stack.peek(), ret)
        self.stack.push(parent)

    def endElementNS(self, (uri, localname), qname):
        self.stack.pop()

    def characters(self, ch):
        if ProcessXML.normalize(ch) != '':
            child = QListViewItem(self.stack.peek(), ProcessXML.normalize(ch))

    def error(self, exception):
        import sys
        sys.stderr.write("\%s\n" % exception)
#}}}


class MainWin (KMainWindow):
#{{{
    STATUSBAR_LEFT   = 1
    STATUSBAR_MIDDLE = 2
    STATUSBAR_RIGHT  = 3

    defaultParser = ProcessXML

    def __init__(self, *args):
#{{{
        apply(KMainWindow.__init__, (self,) + args)

        self.setCaption(args[1].__str__())
        self.setGeometry(QRect(0, 0, 640, 500))

        self.initActions ()
        self.initMenus ()
        self.initToolBar ()
        self.initStatusBar ()

        # setup the tree
        self.tree     = KListView(self, "tree")
        self.setCentralWidget(self.tree)

        self.tree.addColumn(i18n ("XML Tree"))
        self.tree.setRootIsDecorated(True)
        self.tree.setVScrollBarMode(QScrollView.AlwaysOn)

        # XXX: debugging stuff
        #self.xml = "foo.xml"
        self.loadXML()
#}}}

    def loadXML(self):
        if hasattr(self, "xml") and self.xml != None:
            self.tree.clear()
            self.setCaption(self.name() + " " + self.xml.__str__())
            parser = MainWin.defaultParser(self.tree, self.xml)
            parser.parse()

    def initActions(self):
#{{{
        # "File" menu items
        self.openAction   = KStdAction.open (self.slotOpen, self.actionCollection ())
        self.quitAction   = KStdAction.quit (self.close, self.actionCollection ())

        # "Edit" menu items
        #self.findAction     = KStdAction.find (self.slotFind, self.actionCollection ())
        #self.findNextAction = KStdAction.findNext (self.slotFindNext, self.actionCollection ())
        #self.specialAction  = KAction (i18n ("Special"), KShortcut.null (), self.slotSpecial, self.actionCollection (), "special")

        # Demo menu items

        # KToggleAction has an isChecked member and emits the "toggle" signal
        #self.toggle1Action  = KToggleAction ("Toggle 1")
        #self.toggle2Action  = KToggleAction ("Toggle 2", KShortcut.null (), self.slotToggle2, None)

        # A separator - create once/use everywhere
        self.separateAction = KActionSeparator ()


        icons = KIconLoader ()
        iconExpand = QIconSet (icons.loadIcon ("view_tree", KIcon.Toolbar))
        iconCollapse = QIconSet (icons.loadIcon ("view_sidetree", KIcon.Toolbar))

        # expand, collapse
        self.expandAllAction = KAction(i18n("&Expand all"), KShortcut.null() , self.slotExpandAll, self.actionCollection(), "expandall")
        self.expandAllAction.setIconSet(iconExpand)
        self.collapseAllAction = KAction(i18n("&Collapse all"), KShortcut.null() , self.slotCollapseAll, self.actionCollection(), "collapseall")
        self.collapseAllAction.setIconSet(iconCollapse)
#}}}

    def initMenus(self):
#{{{
        fileMenu = QPopupMenu (self)
        self.openAction.plug (fileMenu)
        #fileMenu.insertSeparator ()
        self.quitAction.plug (fileMenu)
        self.menuBar().insertItem (i18n ("&File"), fileMenu)

        #helpMenu = self.helpMenu ("")
        #self.menuBar ().insertItem (i18n ("&Help"), helpMenu)
#}}}

    def initStatusBar(self):
#{{{
        self.statusBar ().insertItem ("", MainWin.STATUSBAR_LEFT, 1000, True)
        self.statusBar ().insertItem ("", MainWin.STATUSBAR_MIDDLE, 1000, True)
        self.statusBar ().insertItem ("", MainWin.STATUSBAR_RIGHT, 1000, True)
#}}}    

    def initToolBar(self):
#{{{
        self.openAction.plug (self.toolBar())
        self.separateAction.plug(self.toolBar())
        self.expandAllAction.plug(self.toolBar())
        self.collapseAllAction.plug(self.toolBar())

#        stretchlbl = QLabel ("", self.toolBar ())
#        self.toolBar ().setStretchableWidget (stretchlbl)
#}}}

#--------------------   slots   --------------------
    def slotOpen(self, id = -1):
        defaultfilename = ""
        filename = KFileDialog.getOpenFileName(defaultfilename, "*.xml", self, "Save to file")
        if not filename.isEmpty() :
            self.xml = filename
            self.loadXML()

    def notImpl(self, item):
        self.statusBar().changeItem("%s not implemented" % item, MainWin.STATUSBAR_LEFT)
        KMessageBox.error(self, "%s not implemented" % item, "Not Implemented")
        self.statusBar().changeItem("", MainWin.STATUSBAR_LEFT)

    def slotExpandAll(self):
        self.expandCollapse(True)
    
    def slotCollapseAll(self):
        self.expandCollapse(False)
    
    def expandCollapse(self, arg):
        it = QListViewItemIterator(self.tree)
        while it.current():
            it.current().setOpen(arg)
            it += 1
 #}}}   


#-------------------- main ------------------------------------------------

description = "iKsml - XML Viewer for KDE"
version     = "0.1"
aboutData   = KAboutData ("", "",\
    version, description, KAboutData.License_GPL,\
    "(C) 2007 Michal Vyskocil")

#aboutData.addAuthor ("author1", "whatever they did", "email@somedomain")
#aboutData.addAuthor ("author2", "they did something else", "another@email.address")

KCmdLineArgs.init (sys.argv, aboutData)

#KCmdLineArgs.addCmdLineOptions ([("--file", "XML file to open")])

app = KApplication ()
mainWindow = MainWin (None, "iKsml " + version)
mainWindow.show()
app.exec_loop()
